// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
import { GraphQLInterfaceType } from 'graphql';
import { getObjectTypeFromClass, GRAPHQL_OBJECT_TYPE_CONFIG_MAP, CLASSES, CONTEXT_INJECTOR_FACTORY_MAP, GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP, GRAPHQL_OBJECT_TYPE_MAP } from './object-type';
export function InterfaceType(config = {}) {
    return target => {
        const classes = Reflect.getMetadata(CLASSES, target) || [];
        classes.push(target);
        Reflect.defineMetadata(CLASSES, classes, target);
        if (config.injector) {
            CONTEXT_INJECTOR_FACTORY_MAP.set(target, config.injector);
        }
        const existingGraphQLObjectTypeQueue = GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(CLASSES, target);
            const classIndex = classes.indexOf(target);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass);
            }
            const existingConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target) || superClassConfig;
            GRAPHQL_OBJECT_TYPE_MAP.set(target, new GraphQLInterfaceType(Object.assign({ name: config.name || target.name, resolveType: function (root, ...args) {
                    const fieldType = config.resolveType.call(root, root, ...args);
                    return getObjectTypeFromClass(fieldType) || fieldType;
                } }, existingConfig)));
        });
        GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target, existingGraphQLObjectTypeQueue);
        return target;
    };
}
//# sourceMappingURL=interface-type.js.map