import { GraphQLObjectType, GraphQLObjectTypeConfig, GraphQLInputType } from 'graphql';
import { Type, AnyType, MaybeArray } from './common';
export declare const CLASSES = "classes";
export declare const PROPERTY_KEYS = "property-keys";
export declare const GRAPHQL_OBJECT_TYPE_CONFIG_MAP: WeakMap<Function, GraphQLObjectTypeConfig<any, any>>;
export declare const GRAPHQL_OBJECT_TYPE_MAP: WeakMap<Function, GraphQLObjectType<any, any>>;
export declare const GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP: WeakMap<Function, (() => void)[]>;
export declare const CONTEXT_INJECTOR_FACTORY_MAP: WeakMap<Function, ContextInjectorFactory<any>>;
export interface ArgDecoratorConfig<TResult> {
    type?: Type<TResult> | GraphQLInputType | object;
    nullable?: boolean;
}
export declare function Arg<TSource, TContext, TResult>(argumentName: string, config?: ArgDecoratorConfig<TResult>): ParameterDecorator;
export interface FieldDecoratorConfig {
    name?: string;
    nullable?: boolean;
}
export declare function Field<TSource, TContext, TArgs, TResult>(typeFactory?: (type: void) => Type<TResult> | GraphQLObjectType | AnyType | unknown, config?: FieldDecoratorConfig): (target: TSource, propertyKey: string) => void;
export declare function Inject(serviceIdentifier?: any): PropertyDecorator;
export interface Injector {
    get<T>(serviceIdentifier: any): T;
}
export declare type ContextInjectorFactory<TContext> = (context: TContext) => Injector | Injector;
export interface ObjectTypeDecoratorConfig<TResult, TContext> {
    name?: string;
    implements?: MaybeArray<Type<TResult> | GraphQLObjectType | AnyType | unknown>;
    injector?: ContextInjectorFactory<TContext>;
}
export declare function ObjectType<TSource, TResult, TContext = any>(config?: ObjectTypeDecoratorConfig<TResult, TContext>): ClassDecorator;
export declare function getObjectTypeFromClass<T>(target: Type<T> | unknown): any;
