// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
import { GraphQLObjectType, GraphQLList, GraphQLNonNull } from 'graphql';
import { DESIGN_PARAMTYPES, DESIGN_TYPE, DESIGN_RETURNTYPE } from './common';
import { getScalarTypeFromClass } from './scalar-type';
import { getInputTypeFromClass } from './input-object-type';
import { asArray } from '../utils/helpers';
export const CLASSES = 'classes';
export const PROPERTY_KEYS = 'property-keys';
export const GRAPHQL_OBJECT_TYPE_CONFIG_MAP = new WeakMap();
export const GRAPHQL_OBJECT_TYPE_MAP = new WeakMap();
export const GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP = new WeakMap();
export const CONTEXT_INJECTOR_FACTORY_MAP = new WeakMap();
export function Arg(argumentName, config) {
    return (target, propertyKey, parameterIndex) => {
        const existingGraphQLObjectTypeQueue = GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(CLASSES, target.constructor);
            const classIndex = classes.indexOf(target.constructor);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass);
            }
            const existingConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target.constructor) || superClassConfig;
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].args = existingConfig.fields[fieldName].args || {};
            existingConfig.fields[fieldName].args.__defineGetter__(argumentName, () => {
                const argumentType = (config && config.type) || Reflect.getMetadata(DESIGN_PARAMTYPES, target, propertyKey)[parameterIndex];
                const argumentGraphQLInputType = getInputTypeFromClass(argumentType) || getScalarTypeFromClass(argumentType) || argumentType;
                if (config && 'nullable' in config && !config.nullable) {
                    return {
                        type: new GraphQLNonNull(argumentGraphQLInputType)
                    };
                }
                else {
                    return {
                        type: argumentGraphQLInputType
                    };
                }
            });
            GRAPHQL_OBJECT_TYPE_CONFIG_MAP.set(target.constructor, existingConfig);
        });
        GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target.constructor, existingGraphQLObjectTypeQueue);
    };
}
export function Field(typeFactory, config) {
    return (target, propertyKey) => {
        const existingGraphQLObjectTypeQueue = GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(CLASSES, target.constructor);
            const classIndex = classes.indexOf(target.constructor);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass) || {};
            }
            const existingConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target.constructor) || superClassConfig;
            const fieldName = (config && config.name) || propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].__defineGetter__('type', () => {
                const fieldType = typeFactory ? typeFactory() : (typeof target[propertyKey] === 'function' ? Reflect.getMetadata(DESIGN_RETURNTYPE, target, propertyKey) : Reflect.getMetadata(DESIGN_TYPE, target, propertyKey));
                const graphQLType = getObjectTypeFromClass(fieldType) || getScalarTypeFromClass(fieldType) || fieldType;
                if (config && 'nullable' in config && !config.nullable) {
                    return new GraphQLNonNull(graphQLType);
                }
                else {
                    return graphQLType;
                }
            });
            if (typeof target[propertyKey] === 'function') {
                existingConfig.fields[fieldName].resolve = ((root, args, context) => {
                    // If 3rd party DI container is defined
                    if (CONTEXT_INJECTOR_FACTORY_MAP.has(target.constructor)) {
                        const contextInjectorFactoryDefinition = CONTEXT_INJECTOR_FACTORY_MAP.get(target.constructor);
                        let injector;
                        if (typeof contextInjectorFactoryDefinition === 'function') {
                            injector = contextInjectorFactoryDefinition(context);
                        }
                        else if (typeof contextInjectorFactoryDefinition === 'object') {
                            injector = contextInjectorFactoryDefinition;
                        }
                        const keys = Reflect.getMetadata(PROPERTY_KEYS, target.constructor) || [];
                        for (const key of keys) {
                            if (Reflect.getMetadata(DESIGN_TYPE, target, key)) {
                                root = root || {};
                                Object.defineProperty(root, key, {
                                    get() {
                                        const serviceIdentifier = Reflect.getMetadata(DESIGN_TYPE, target, key);
                                        return injector.get(serviceIdentifier);
                                    }
                                });
                            }
                        }
                    }
                    return target[propertyKey].call(root, ...Object['values'](args));
                }); // TODO: NOT SAFE
            }
            GRAPHQL_OBJECT_TYPE_CONFIG_MAP.set(target.constructor, existingConfig);
        });
        GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target.constructor, existingGraphQLObjectTypeQueue);
    };
}
export function Inject(serviceIdentifier) {
    return (target, propertyKey, index) => {
        const allDependencies = Reflect.getMetadata(DESIGN_PARAMTYPES, target) || [];
        const propertyKeys = Reflect.getMetadata(PROPERTY_KEYS, target.constructor || target) || [];
        if (typeof propertyKey === 'undefined') {
            if (typeof index !== 'undefined') {
                allDependencies[index] = serviceIdentifier;
            }
        }
        else {
            const designType = serviceIdentifier || Reflect.getMetadata(DESIGN_TYPE, target, propertyKey);
            Reflect.defineMetadata(DESIGN_TYPE, designType, target, propertyKey);
            propertyKeys.push(propertyKey);
        }
        Reflect.defineMetadata(DESIGN_PARAMTYPES, allDependencies, target);
        Reflect.defineMetadata(PROPERTY_KEYS, propertyKeys, target.constructor || target);
        return target;
    };
}
export function ObjectType(config = {}) {
    return target => {
        const classes = Reflect.getMetadata(CLASSES, target) || [];
        classes.push(target);
        Reflect.defineMetadata(CLASSES, classes, target);
        if (config.injector) {
            CONTEXT_INJECTOR_FACTORY_MAP.set(target, config.injector);
        }
        const existingGraphQLObjectTypeQueue = GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(CLASSES, target);
            const classIndex = classes.indexOf(target);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass);
            }
            const existingConfig = GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target) || superClassConfig;
            GRAPHQL_OBJECT_TYPE_MAP.set(target, new GraphQLObjectType(Object.assign({ name: config.name || target.name, interfaces: asArray(config.implements).map(interfaceType => getObjectTypeFromClass(interfaceType) || interfaceType) }, existingConfig)));
        });
        GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target, existingGraphQLObjectTypeQueue);
        return target;
    };
}
export function getObjectTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getObjectTypeFromClass(target[0]);
        return elementType && new GraphQLList(elementType);
    }
    if (!GRAPHQL_OBJECT_TYPE_CONFIG_MAP.has(target) &&
        GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.has(target)) {
        const existingGraphQLObjectTypeQueue = GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target);
        existingGraphQLObjectTypeQueue.forEach(fn => fn());
    }
    return GRAPHQL_OBJECT_TYPE_MAP.get(target);
}
//# sourceMappingURL=object-type.js.map