"use strict";
// tslint:disable-next-line:no-reference
/// <reference path="../../node_modules/reflect-metadata/index.d.ts" />
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const common_1 = require("./common");
const scalar_type_1 = require("./scalar-type");
const input_object_type_1 = require("./input-object-type");
const helpers_1 = require("../utils/helpers");
exports.CLASSES = 'classes';
exports.PROPERTY_KEYS = 'property-keys';
exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP = new WeakMap();
exports.GRAPHQL_OBJECT_TYPE_MAP = new WeakMap();
exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP = new WeakMap();
exports.CONTEXT_INJECTOR_FACTORY_MAP = new WeakMap();
function Arg(argumentName, config) {
    return (target, propertyKey, parameterIndex) => {
        const existingGraphQLObjectTypeQueue = exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(exports.CLASSES, target.constructor);
            const classIndex = classes.indexOf(target.constructor);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass);
            }
            const existingConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target.constructor) || superClassConfig;
            const fieldName = propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].args = existingConfig.fields[fieldName].args || {};
            existingConfig.fields[fieldName].args.__defineGetter__(argumentName, () => {
                const argumentType = (config && config.type) || Reflect.getMetadata(common_1.DESIGN_PARAMTYPES, target, propertyKey)[parameterIndex];
                const argumentGraphQLInputType = input_object_type_1.getInputTypeFromClass(argumentType) || scalar_type_1.getScalarTypeFromClass(argumentType) || argumentType;
                if (config && 'nullable' in config && !config.nullable) {
                    return {
                        type: new graphql_1.GraphQLNonNull(argumentGraphQLInputType)
                    };
                }
                else {
                    return {
                        type: argumentGraphQLInputType
                    };
                }
            });
            exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.set(target.constructor, existingConfig);
        });
        exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target.constructor, existingGraphQLObjectTypeQueue);
    };
}
exports.Arg = Arg;
function Field(typeFactory, config) {
    return (target, propertyKey) => {
        const existingGraphQLObjectTypeQueue = exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target.constructor) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(exports.CLASSES, target.constructor);
            const classIndex = classes.indexOf(target.constructor);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass) || {};
            }
            const existingConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target.constructor) || superClassConfig;
            const fieldName = (config && config.name) || propertyKey;
            existingConfig.fields = existingConfig.fields || {};
            existingConfig.fields[fieldName] = existingConfig.fields[fieldName] || {};
            existingConfig.fields[fieldName].__defineGetter__('type', () => {
                const fieldType = typeFactory ? typeFactory() : (typeof target[propertyKey] === 'function' ? Reflect.getMetadata(common_1.DESIGN_RETURNTYPE, target, propertyKey) : Reflect.getMetadata(common_1.DESIGN_TYPE, target, propertyKey));
                const graphQLType = getObjectTypeFromClass(fieldType) || scalar_type_1.getScalarTypeFromClass(fieldType) || fieldType;
                if (config && 'nullable' in config && !config.nullable) {
                    return new graphql_1.GraphQLNonNull(graphQLType);
                }
                else {
                    return graphQLType;
                }
            });
            if (typeof target[propertyKey] === 'function') {
                existingConfig.fields[fieldName].resolve = ((root, args, context) => {
                    // If 3rd party DI container is defined
                    if (exports.CONTEXT_INJECTOR_FACTORY_MAP.has(target.constructor)) {
                        const contextInjectorFactoryDefinition = exports.CONTEXT_INJECTOR_FACTORY_MAP.get(target.constructor);
                        let injector;
                        if (typeof contextInjectorFactoryDefinition === 'function') {
                            injector = contextInjectorFactoryDefinition(context);
                        }
                        else if (typeof contextInjectorFactoryDefinition === 'object') {
                            injector = contextInjectorFactoryDefinition;
                        }
                        const keys = Reflect.getMetadata(exports.PROPERTY_KEYS, target.constructor) || [];
                        for (const key of keys) {
                            if (Reflect.getMetadata(common_1.DESIGN_TYPE, target, key)) {
                                root = root || {};
                                Object.defineProperty(root, key, {
                                    get() {
                                        const serviceIdentifier = Reflect.getMetadata(common_1.DESIGN_TYPE, target, key);
                                        return injector.get(serviceIdentifier);
                                    }
                                });
                            }
                        }
                    }
                    return target[propertyKey].call(root, ...Object['values'](args));
                }); // TODO: NOT SAFE
            }
            exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.set(target.constructor, existingConfig);
        });
        exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target.constructor, existingGraphQLObjectTypeQueue);
    };
}
exports.Field = Field;
function Inject(serviceIdentifier) {
    return (target, propertyKey, index) => {
        const allDependencies = Reflect.getMetadata(common_1.DESIGN_PARAMTYPES, target) || [];
        const propertyKeys = Reflect.getMetadata(exports.PROPERTY_KEYS, target.constructor || target) || [];
        if (typeof propertyKey === 'undefined') {
            if (typeof index !== 'undefined') {
                allDependencies[index] = serviceIdentifier;
            }
        }
        else {
            const designType = serviceIdentifier || Reflect.getMetadata(common_1.DESIGN_TYPE, target, propertyKey);
            Reflect.defineMetadata(common_1.DESIGN_TYPE, designType, target, propertyKey);
            propertyKeys.push(propertyKey);
        }
        Reflect.defineMetadata(common_1.DESIGN_PARAMTYPES, allDependencies, target);
        Reflect.defineMetadata(exports.PROPERTY_KEYS, propertyKeys, target.constructor || target);
        return target;
    };
}
exports.Inject = Inject;
function ObjectType(config = {}) {
    return target => {
        const classes = Reflect.getMetadata(exports.CLASSES, target) || [];
        classes.push(target);
        Reflect.defineMetadata(exports.CLASSES, classes, target);
        if (config.injector) {
            exports.CONTEXT_INJECTOR_FACTORY_MAP.set(target, config.injector);
        }
        const existingGraphQLObjectTypeQueue = exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target) || [];
        existingGraphQLObjectTypeQueue.push(() => {
            const classes = Reflect.getMetadata(exports.CLASSES, target);
            const classIndex = classes.indexOf(target);
            let superClassConfig = {};
            if (classIndex > 0) {
                const superClass = classes[classIndex - 1];
                getObjectTypeFromClass(superClass);
                superClassConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(superClass);
            }
            const existingConfig = exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.get(target) || superClassConfig;
            exports.GRAPHQL_OBJECT_TYPE_MAP.set(target, new graphql_1.GraphQLObjectType(Object.assign({ name: config.name || target.name, interfaces: helpers_1.asArray(config.implements).map(interfaceType => getObjectTypeFromClass(interfaceType) || interfaceType) }, existingConfig)));
        });
        exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.set(target, existingGraphQLObjectTypeQueue);
        return target;
    };
}
exports.ObjectType = ObjectType;
function getObjectTypeFromClass(target) {
    if (target instanceof Array) {
        const elementType = getObjectTypeFromClass(target[0]);
        return elementType && new graphql_1.GraphQLList(elementType);
    }
    if (!exports.GRAPHQL_OBJECT_TYPE_CONFIG_MAP.has(target) &&
        exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.has(target)) {
        const existingGraphQLObjectTypeQueue = exports.GRAPHQL_OBJECT_TYPE_CONFIG_BUILD_QUEUE_MAP.get(target);
        existingGraphQLObjectTypeQueue.forEach(fn => fn());
    }
    return exports.GRAPHQL_OBJECT_TYPE_MAP.get(target);
}
exports.getObjectTypeFromClass = getObjectTypeFromClass;
//# sourceMappingURL=object-type.js.map